/* adjustment.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_ADJUSTMENT_HH
#define GLTK_ADJUSTMENT_HH
#include "sigc.hh"
namespace gltk {
	/// value with a defined range and two sizes of increments
	class Adjustment
	{
		sigc::signal<void> changed;
		sigc::signal<void> value_changed;
	protected:
		// internal state
		double value; ///< the value
		double lower; ///< lower bound
		double upper; ///< upper bound
		double step_increment; ///< step increment
		double page_increment; ///< page increment
		double page_size; ///< size of a page
		virtual void on_value_changed() {};
	public:
		/// initialied with defined parameters
		Adjustment(
			double value, ///< set initial #value
			double lower, ///< set #lower
			double upper, ///< set #upper
			double step_increment = 1, ///< set #step_increment
			double page_increment = 10, ///< set #page_increment
			double page_size = 0 ///< set #page_size
		);
		static Adjustment * create(
			double value,
			double lower,
			double upper,
			double step_increment = 1,
			double page_increment = 10,
			double page_size = 0
		);
		/// change the parameter of the Adjustment
		void configure(
			double value,
			double lower,
			double upper,
			double step_increment=1,
			double page_increment=10,
			double page_size=0
		);
		double get_value() const;
		void set_value(double value);
		void step_up();
		void step_down();
		void page_up();
		void page_down();
		void set_step_increment(double step_increment);
		void set_page_increment(double page_increment);
		double get_upper() const {return upper;}
		void set_upper (double upper);
		double get_lower() const {return lower;}
		void set_lower (double lower);
		double get_page_size() const {return page_size;}
		void set_page_size(double ps);
		/// get the signal on value change
		sigc::signal<void> & signal_changed();
		sigc::signal<void> & signal_value_changed();
	};
}
#endif
