/* box.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_BOX_HH
#define GLTK_BOX_HH
#include "orientable.hh"
#include "container.hh"
#include "packoptions.hh"
namespace gltk {
	/** A Container that lines up its children.
	    This is a linear layout of Widget%s. The Box model decomposes
	    a Widget layout into a hierachy of horizontal and vertical lineups. */
	class Box :
		public Orientable,
		public Container
	{
	private:
		void on_configure_event();

		void get_preferred_width(int & min, int & nat) const;
		void get_preferred_height(int & min, int & nat) const;
		void get_preferred_width_for_height(int height, int & min, int & nat) const;
		void get_preferred_height_for_width(int width, int & min, int & nat) const;
	protected:
		unsigned border_width;
		int spacing; ///< natural spacing between child Widget%s
		bool homogeneous;
	public:
		/// initialize the Box for given orientation and spacing
		Box(
			Orientation orientabtion = ORIENTATION_HORIZONTAL, ///< set orientation
			int spacing = 0 ///< set spacing between widgets
		);
		void pack_start(Widget * child); ///< add a child Widget to the left or top end of the lineup
		void pack_start(Widget & child, PackOptions options = PACK_EXPAND_WIDGET, unsigned padding = 0) {pack_start(& child);}

		void set_border_width(unsigned border_width);
	};

	/// vertical Box
	class VBox :
		public Box
	{
	public:
		VBox(bool homogeneous = false, int spacing = 0) :
			Box(ORIENTATION_VERTICAL, spacing)
		{}
	};

	/// horizontal Box
	class HBox :
		public Box
	{
	public:
		HBox(bool homogeneous = false, int spacing = 0) :
			Box(ORIENTATION_HORIZONTAL, spacing)
		{}
	};
}
#endif
