/* checkbox.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_CHECKBOX_HH
#define GLTK_CHECKBOX_HH
#include "focusable.hh"
#include "sigc.hh"
namespace gltk {
	/** a checkbox that can be toggled on and off
	    \image html CheckBox.png */
	class CheckBox :
		virtual public Focusable
	{
		sigc::signal<void> toggled;
		bool active;

		// overrides
		void on_draw();

		void get_preferred_width(int & min, int & nat) const;
		void get_preferred_height(int & min, int & nat) const;

	protected:
		void on_button_event(int button, int state, int x, int y);
		bool on_key_event(unsigned char key);
		/// Draw checkbox at the center of specified area.
		void draw_checkbox(int x, int y, int w, int h);
		static const int checkbox_size;
		static const int checkbox_area;
	public:
		CheckBox();
		sigc::signal<void> & signal_toggled(); ///< signal for toggling event
		bool get_active() const; ///< get the state of the toggle
		void set_active(bool val = true); ///< set the state of the toggle
	};
}
#endif
