/* checkbutton.cc
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#include "checkbutton.hh"
#include "label.hh"
#include "internal.hh"
using namespace gltk;

void CheckButton::on_configure_event()
{
	Bin::on_configure_event();
	if (child) {
		int ch = child->get_height();
		child->move_allocate(min_of(checkbox_area, get_width()), (get_height() - ch) / 2);
	}
}

void CheckButton::on_draw()
{
	int w = get_width();
	int h = get_height();
	draw_focus_box();
	w = min_of(w, checkbox_area);
	draw_checkbox(0, 0, w, h);
	draw_children();
}

void CheckButton::get_preferred_width(int & min, int & nat) const
{
	Bin::get_preferred_width(min, nat);
	min = checkbox_size; // child is dispensable
}

void CheckButton::get_preferred_height(int & min, int & nat) const
{
	Bin::get_preferred_height(min, nat);
	min = checkbox_size; // child is dispensable
	nat = max_of(nat, checkbox_area);
}

void CheckButton::get_preferred_width_for_height(int height, int & min, int & nat) const
{
	Bin::get_preferred_width_for_height(height, min, nat);
	min = checkbox_size;
}

void CheckButton::get_preferred_height_for_width(int width, int & min, int & nat) const
{
	Bin::get_preferred_height_for_width(width, min, nat);
	min = checkbox_size;
	nat = max_of(nat, checkbox_area);
}

void CheckButton::on_button_event(int button, int state, int x, int y)
{
	CheckBox::on_button_event(button, state, x, y);
}

bool CheckButton::on_key_event(unsigned char key)
{
	return CheckBox::on_key_event(key);
}

bool CheckButton::focus_make()
{
	return CheckBox::focus_make();
}

bool CheckButton::focus_next()
{
	return CheckBox::focus_next();
}

void CheckButton::focus_end()
{
	CheckBox::focus_end();
}

CheckButton::CheckButton(std::string const & label, bool mnemonic) :
	LabelBin(label)
{
	set_reserved(checkbox_area + 1, 2); // 2 for focus indicator
}
