/* comboboxtext.cc
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#include "comboboxtext.hh"
#include "glut_inc.hh"
#include "internal.hh"
#include <iostream>
using namespace gltk;

void ComboBoxText::on_draw()
{
	int w = get_width();
	int h = get_height();
	glut_draw_raised_box(0, 0, w, h);
	int bw = glut_raised_box_width();
	glColor3f(0.9, 0.9, 0.9);
	glBegin(GL_QUADS);
	glVertex2i(bw, bw);
	glVertex2i(w - bw, bw);
	glVertex2i(w - bw, h - bw);
	glVertex2i(bw, h - bw);
	glEnd();

	// draw string
	int y = h / 2;
	int x = bw + 1;
	glColor3f(0.0, 0.0, 0.0);
	glRasterPos2i(x, y);
	draw_text(ALIGN_START, ALIGN_CENTER, max_of(1, get_width() - bw * 2 - 14));

	// draw arrows
	y = h / 2;
	x = w - bw - 1;
	glBegin(GL_TRIANGLES);
	glVertex2f(x, y + 1);
	glVertex2f(x - 8, y + 1);
	glVertex2f(x - 4, y + 5);
	glVertex2f(x, y - 1);
	glVertex2f(x - 8, y - 1);
	glVertex2f(x - 4, y - 5);
	glEnd();
}

void ComboBoxText::get_preferred_height(int & min, int & nat) const
{
	int h = text_height();
	min = h + glut_raised_box_width() * 2;
	min = max_of(min, 12);
	nat = min + 6;
}

void ComboBoxText::get_preferred_width(int & min, int & nat) const
{
	int w = 0;
	for (std::vector<std::string>::const_iterator i = menu.begin(); i != menu.end(); i ++) {
		w = max_of(w, text_width(* i));
	}
	min = w + glut_raised_box_width() * 2 + 14;
	nat = min;
}

void ComboBoxText::on_menu_select(int item)
{
	set_text(menu[item]);
	active_item = item;
	changed.emit();
	queue_draw();
}

ComboBoxText::ComboBoxText() :
	active_item(0)
{
}

void ComboBoxText::append(std::string const & text)
{
	menu.push_back(text);
	if (active_item == int(menu.size()) - 1) {
		set_text(text);
		queue_draw();
	}
	release_menu();
}

std::string ComboBoxText::get_active_text() const
{
	if (active_item >= 0 && active_item < (int) menu.size()) return menu[active_item];
	return std::string();
}

void ComboBoxText::remove_all()
{
	menu.clear();
	release_menu();
	active_item = 0;
}

void ComboBoxText::set_active_text(std::string const & text)
{
	for (int i = 0; i < (int) menu.size(); i ++)
	if (menu[i] == text) {
		active_item = i;
	}
	set_text(text);
	queue_draw();
}
