/* comboboxtext.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_COMBOBOXTEXT_HH
#define GLTK_COMBOBOXTEXT_HH
#include "popup.hh"
#include "textdraw.hh"
#include "sigc.hh"
#include <vector>
namespace gltk {
	/** A Widget used to choose from a list of text items.
	    \image html ComboBoxText.png
	    The popup menu of GLUT uses underlying window system.
	    Its look and feel is beyond control. */
	class ComboBoxText :
		public virtual TextDraw,
		public virtual Popup
	{
		sigc::signal<void> changed;
		void on_draw();
		void get_preferred_height(int & min, int & nat) const;
		void get_preferred_width(int & min, int & nat) const;
		void on_menu_select(int item);
		int active_item;
	public:
		ComboBoxText();
		void append(std::string const & text); ///< appends text to the list of strings
		std::string get_active_text() const; ///< return the currently active text item
		void remove_all(); ///< remove all items
		void set_active_text(std::string const & text); ///< set the default active text
		sigc::signal<void> & signal_changed() {return changed;} ///< signal when the active item is changed
	};
}
#endif
