/* container.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_CONTAINER_HH
#define GLTK_CONTAINER_HH
#include "widget.hh"
#include <vector>
namespace gltk {
	/** A Container holds child widget(s).

	    It should be able to iterate over all widget(s).
	    In GTK+, this can be a Bin or a Box
	*/
	class Container :
		public virtual Widget
	{
		// overrides
		void on_set_window(Window * window);
		void on_realize();
		void on_unrealize();
		void on_draw();
		void on_button_event(int button, int state, int x, int y);
		void on_motion_event(int x, int y);
		void on_passive_event(int x, int y);
		bool on_key_event(unsigned char key);
		bool on_special_event(int key);
		void on_entry_event(bool entered);
		bool focus_make();
		bool focus_next();
		void focus_end();
	protected:
		// tools
		Widget * get_child(int x, int y);
		std::vector<Widget *> child_list;
		Widget * grabber;
		Widget * focus;
		Widget * tracker;
		virtual void on_add(Widget *) {}
		virtual void on_remove(Widget *) {}

		void draw_children();
		void set_children_window(Window * window);
		void realize_children();
		void unrealize_children();
	public:
		Container();
		~Container();
		void add(Widget * child);
		void add(Widget & child) {add(& child);}
		void remove(Widget * child);
		void show_all();
		void show_all_children(bool recursive = true);
		void set_grabber(Widget * child, bool grab = true);
		virtual bool focus_on(Widget * child); // child requested, child guarantees taking the focus
	};
}
#endif
