/* entry.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_ENTRY_HH
#define GLTK_ENTRY_HH
#include "widget.hh"
#include "sigc.hh"
#include <string>
namespace gltk {
	/** Single line text input.
	    This Widget shows a one-line text field and allows manipulation of
	    the text.
	    \image html Entry.png */
	class Entry :
		public virtual Widget
	{
		// text editor
		std::string text;
		size_t max_length;
		bool focused;
		bool editable;
		bool visibility;

		size_t cursor; // cursor position
		size_t vstart; // start of visible text
		size_t mark; // marked position
		bool marked;

		void * font;

		int width_chars; // width in # of characters

		// overrides
		void on_draw();
		void on_button_event(int button, int state, int x, int y);
		void on_motion_event(int x, int y);
		bool on_key_event(unsigned char key);
		bool on_special_event(int key);

		bool focus_make();
		void focus_end();

		// internal
		void adjust_vstart();
		int find_x(size_t p);
		size_t find_position(int x);
		void delete_marked();
		int bw; // border width
		int as; // arrow space
		bool end_arrow; // end arrow visible?

		// signals
		sigc::signal<void> activate;
		sigc::signal<void> stop;
	public:
		Entry();

		void get_preferred_height(int & min, int & nat) const;
		void get_preferred_width(int & min, int & nat) const;

		void set_text(std::string const &); ///< set the content of input
		std::string const & get_text() const; ///< get the content of input
		size_t get_text_length() const; ///< return length of input
		void select_all();
		void select_none();
		void set_max_length(size_t length);
		void set_width_chars(int n_chars);
		void set_editable(bool v = true); ///< enable/disable editing
		void set_visibility(bool v = true); ///< obscure the charactes
		sigc::signal<void> & signal_activate(); ///< signal when text is enterred
		sigc::signal<void> & signal_stop(); ///< signal when Entry lost focus
	};
}
#endif
