/* glarea.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_GLAREA_HH
#define GLTK_GLAREA_HH
#include "glutwindow.hh"
#include "widget.hh"
namespace gltk {
	/** A Widget with its own GL context.
	    glArea has an independent GL context allowing 3D drawing.
	    The underlying GLUT subwindow is created when glArea is configured
	    and shown. */
	class glArea :
		public glutWindow,
		public Widget
	{
		void on_display_func();
		void on_reshape_func(int width, int height);
		// override Widget
		void on_draw();
		void on_unrealize() /*override*/;
		void on_configure_event() /*final*/; // realize after configured.
		void on_show() /*final*/;
		void on_hide() /*final*/;

		// internal parameter
		int area_width;
		int area_height;
		// geometry overrides
		void get_preferred_width(int & min, int & nat) const;
		void get_preferred_height(int & min, int & nat) const;

		void make_realized();
	protected:
		virtual void gl_setup() {}
		virtual void gl_new_size(int w, int h) {}
		virtual void gl_draw() {}
		virtual void gl_button_event(int button, int state, int x, int y) {}
		virtual void gl_motion_event(int x, int y) {}
	public:
		glArea();
		~glArea();
		void set_size(int width, int height);
	};
}
#endif
