/* glutwindow.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_GLUTWINDOW_HH
#define GLTK_GLUTWINDOW_HH
#include <string>
namespace gltk {
	/** Abstraction of underlying system window.
	    This is the start of abstraction layer over a GLUT window or subwindow.
	    It provides handles for all callbacks from GLUT. */
	class glutWindow
	{
		friend class _glutWindow;
		// overrides
		int win_id; ///< the GLUT window id of a realized Window
		unsigned event_mask;
	protected:
		virtual void on_display_func() {} ///< called to implement draw
		virtual void on_reshape_func(int width, int height) {} ///< called when size is given
		virtual void on_mouse_func(int button, int state, int x, int y) {} ///< called for mouse button
		virtual void on_motion_func(int x, int y) {} ///< called for grabbed movement
		virtual void on_passive_func(int x, int y) {} ///< called for non-grabbed movement
		virtual void on_keyboard_func(unsigned char key, int x, int y) {} ///< called for key press
		virtual void on_special_func(int key, int x, int y) {} ///< called for special keye
		virtual void on_entry_func(int state) {} ///< called when pointer enter/leave
		enum EventMask {
			EVENT_MASK_NONE = 0,
			DISPLAY_MASK = 1 << 0,
			RESHAPE_MASK = 1 << 1,
			MOUSE_MASK = 1 << 2,
			MOTION_MASK = 1 << 3,
			PASSIVE_MASK = 1 << 4,
			KEYBOARD_MASK = 1 << 5,
			SPECIAL_MASK = 1 << 6,
			ENTRY_MASK = 1 << 7,
			EVENT_MASK_ALL = (1 << 8) - 1
		};

		virtual void on_create() {}
		virtual void on_destroy() {}
		glutWindow();
		virtual ~glutWindow();
		void set_events(unsigned mask); ///< set the events to handle
		void add_events(unsigned mask); ///< set the events to handle
		void create(int parent_win_id, int x, int y, int width, int height); ///< create the GLUT window
		void create(std::string const & name, int width, int height); ///< create the GLUT window
		void destroy(); ///< destroy the GLUT window
		void destroy_all(); ///< exit?
		int get_win_id() const {return win_id;}

		void set_current(); ///< make it the current window
		void move(int x, int y); ///< move the window
		void resize(int width, int height); ///< change the size of mapped window
		void redisplay(); ///< post redisplay
	};
}
#endif
