/* idle.cc
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#include "idle.hh"
#include "glut_inc.hh"
#include "internal.hh"
#include <list>
using namespace gltk;

namespace {
	unsigned idle_unique_id = 0;

	struct IdleEvent
	{
		unsigned idle_id;
		sigc::functor_base * fun;
	};

	std::list<IdleEvent> idle_event_list;

	void idle_func()
	{
		if (idle_event_list.size()) {
			IdleEvent e = idle_event_list.front();
			idle_event_list.pop_front();
			if (sigc::functor0<bool> * f = dynamic_cast<sigc::functor0<bool> *>(e.fun)) {
				if (f->call()) { // repeat
					idle_event_list.push_back(e);
					return;
				}
			}
			else if (sigc::functor0<void> * f = dynamic_cast<sigc::functor0<void> *>(e.fun)) {
				f->call();
			}
			if (! idle_event_list.size()) glutIdleFunc(0);
		}
		else glutIdleFunc(0);
	}

	struct _Idle :
		public sigc::signal_base // just so disconnect() works, the handle_list is by-passed.
	{
		unsigned add_handle(sigc::slot_base const & slot)
		{
			throw Error("add_handle() called on idle_signal!");
		}

		void remove_handle(unsigned hid)
		{
			if (! idle_event_list.size()) return; // nothing to remove
			for (std::list<IdleEvent>::iterator i = idle_event_list.begin(); i != idle_event_list.end(); i ++)
			if (i->idle_id == hid) {
				delete i->fun;
				idle_event_list.erase(i);
				break;
			}
			if (! idle_event_list.size()) {
				glutIdleFunc(0);
			}
		}
	} idle;
}

sigc::connection SignalIdle::connect(sigc::slot0<bool> const & slot)
{
	IdleEvent e;
	e.idle_id = idle_unique_id;
	e.fun = slot.fun->dup();
	idle_event_list.push_back(e);
	glutIdleFunc(& idle_func);
	return sigc::connection(& idle, unsigned(idle_unique_id ++));
}

void SignalIdle::connect_once(sigc::slot0<void> const & slot)
{
	IdleEvent e;
	e.idle_id = idle_unique_id;
	e.fun = slot.fun->dup();
	idle_event_list.push_back(e);
	glutIdleFunc(& idle_func);
	idle_unique_id ++;
}
