/* popup.cc
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#include "popup.hh"
#include "glut_inc.hh"
#include "window.hh"
#include <iostream>
#include <map>
using namespace gltk;

std::map<int, Popup *> popup_list;

void Popup::menu_callback(int item)
{
	Popup & p = * popup_list[glutGetMenu()];
	p.on_menu_select(item);
}

void Popup::menu_status(int status, int x, int y)
{
}

// Maybe I should use glutMenuStatusFunc to get the menu status
void Popup::on_entry_event(bool ent)
{
	get_window()->make_current();
	if (ent) {
		if (menu_id) glutSetMenu(menu_id);
		else {
			menu_id = glutCreateMenu(& menu_callback);
			popup_list[menu_id] = this;
			for (int i = 0; i < (int) menu.size(); i ++) {
				glutAddMenuEntry(menu[i].c_str(), i);
			}
			/* The menu status function somehow does not work...
			 */
			// glutMenuStatusFunc(& menu_status);
		}
		glutAttachMenu(GLUT_LEFT_BUTTON);
		glutSetCursor(GLUT_CURSOR_UP_DOWN);
	}
	else {
		if (menu_id) {
			glutSetMenu(menu_id);
			glutDetachMenu(GLUT_LEFT_BUTTON);
			glutSetCursor(GLUT_CURSOR_INHERIT);
		}
	}
}

void Popup::on_unrealized()
{
	release_menu();
}

void Popup::release_menu()
{
	if (menu_id) {
		popup_list.erase(menu_id);
		glutDestroyMenu(menu_id);
		menu_id = 0;
	}
}

Popup::Popup() :
	menu_id(0)
{}
