/* slider.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_SLIDER_HH
#define GLTK_SLIDER_HH
#include "widget.hh"
#include "adjustment.hh"
#include "orientable.hh"
namespace gltk {
	/** Selecting a value by moving a slider.
	    Slider can be used to select a value for an Adjustment. */
	class Slider :
		public virtual Orientable,
		public virtual Widget
	{
		Adjustment * adjust;
		void on_draw();
		void on_button_event(int button, int state, int x, int y);
		void on_motion_event(int x, int y);

		void get_preferred_width(int & min, int & nat) const;
		void get_preferred_height(int & min, int & nat) const;

		int slider_z0;
		int slider_sz; // size of slider
		bool sliding;
		int sliding_start;
		double sliding_vs;
	public:
		///< construct Slider on given Adjustment and orientation
		Slider(
			Adjustment * adjust = 0,
			Orientation orient = ORIENTATION_HORIZONTAL
		);
		Adjustment * get_adjustment(); ///< return the controlled Adjustment
		void set_value(double value); ///< set the value
		double get_value() const; ///< get the value

		// internal, for tracking the slider
		int get_slider_pos() const;
		int get_slider_size() const;

		sigc::signal<void> & signal_value_changed(); ///< when the value is changed
	};
}
#endif
