/* textdraw.cc
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#include "textdraw.hh"
#include "glut_inc.hh"
#include "internal.hh"
#include <iostream>
#include <algorithm>
using namespace gltk;

void TextDraw::draw_text(Align xl, Align yl, int wlimit)
{
	int x = 0;
	int y = 0;
	switch (xl) {
	case ALIGN_END:
		x = - text_width();
		break;
	case ALIGN_CENTER:
		x = - text_width() / 2;
		break;
	default:
		break;
	}
	switch (yl) {
	case ALIGN_CENTER:
		y -= glut_get_font_height(font) / 2;
	case ALIGN_START:
		y += glut_get_font_ascent(font);
		break;
	case ALIGN_END:
		y = - glut_get_font_descent(font);
		break;
	default:
		break;
	}
	glBitmap(0, 0, 0, 0, x, - y, 0);
	if (wlimit > 0) glut_draw_string(font, text, wlimit);
	else glut_draw_string(font, text);
}

int TextDraw::text_height() const
{
	return glut_get_font_height(font);
}

int TextDraw::text_width() const
{
	return text_width(text);
}

int TextDraw::text_width(std::string const & tx) const
{
	int w = 0;
	for (size_t i = 0; i < tx.size(); i ++) w += glutBitmapWidth(font, tx[i]);
	return w;
}

TextDraw::TextDraw() :
	font(GLUT_BITMAP_HELVETICA_12)
{}

void TextDraw::set_text(std::string const & t)
{
	text = t;
	on_set_text();
}

int TextDraw::get_ascent() const
{
	return glut_get_font_ascent(font);
}
