/* timeout.cc
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#include "timeout.hh"
#include "glut_inc.hh"
#include "internal.hh"
#include <map>
using namespace gltk;

namespace {

	int timer_unique_id = 0;
	struct TimerEvent
	{
		sigc::functor_base * fun;
		unsigned interval;
	};
	std::map<int, TimerEvent> timer_event_list;

	void timer_func(int v)
	{
		std::map<int, TimerEvent>::iterator i = timer_event_list.find(v);
		if (i == timer_event_list.end()) return; // event is gone!
		if (sigc::functor0<bool> * f = dynamic_cast<sigc::functor0<bool> *>(i->second.fun)) {
			if (f->call()) {
				// wants to reconnect
				glutTimerFunc(i->second.interval, & timer_func, v);
				return;
			}
		}
		else if (sigc::functor0<void> * f = dynamic_cast<sigc::functor0<void> *>(i->second.fun)) {
			f->call();
		}
		delete i->second.fun;
		timer_event_list.erase(i);
	}

	struct _Timeout :
		public sigc::signal_base // just so disconnect() works, the handle_list is by-passed.
	{
		unsigned add_handle(sigc::slot_base const & slot)
		{
			throw Error("add_handle() called on timeout_signal!");
		}

		void remove_handle(unsigned hid)
		{
			int v = hid;
			std::map<int, TimerEvent>::iterator i = timer_event_list.find(v);
			if (i == timer_event_list.end()) return;
			delete i->second.fun;
			timer_event_list.erase(i);
		}
	} timeout;
}

sigc::connection SignalTimeout::connect(sigc::slot0<bool> const & slot, unsigned interval)
{
	TimerEvent & e = timer_event_list[timer_unique_id];
	e.fun = slot.fun->dup();
	e.interval = interval;
	glutTimerFunc(interval, & timer_func, timer_unique_id);
	return sigc::connection(& timeout, unsigned(timer_unique_id ++));
}

void SignalTimeout::connect_once(sigc::slot0<void> const & slot, unsigned interval)
{
	TimerEvent & e = timer_event_list[timer_unique_id];
	e.fun = slot.fun->dup();
	e.interval = interval;
	glutTimerFunc(interval, & timer_func, timer_unique_id);
	timer_unique_id ++;
}
