/* window.hh
 * 
 * Copyright (C) 2011 Chun-Chung Chen <cjj@u.washington.edu>
 * 
 * This file is part of gltk.
 * 
 * gltk is free software: you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with gltk.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
#ifndef GLTK_WINDOW_HH
#define GLTK_WINDOW_HH
#include "glutwindow.hh"
#include "bin.hh"
#include <string>
#include <map>
namespace gltk {
	/** a toplevel window.
	    This is a GLUT window that handles all callbacks from GLUT. */
	class Window :
		glutWindow,
		public virtual Bin
	{
		// keep track of visible window count
		static size_t visible_count;
		bool visible;
		bool to_reconf;

		// overrides
		void on_display_func(); ///< called to implement draw
		void on_reshape_func(int width, int height); ///< called when size is given
		void on_mouse_func(int button, int state, int x, int y); ///< called for mouse button
		void on_motion_func(int x, int y); ///< called for grabbed movement
		void on_passive_func(int x, int y); ///< called for non-grabbed movement
		void on_keyboard_func(unsigned char key, int x, int y); ///< called for key press
		void on_special_func(int key, int x, int y); ///< called for special keye

		// overrides
		void on_create();
		void on_destroy();
		void on_add(Widget * widget);

		bool get_focus();

		int default_w; ///< initial width that will be requested
		int default_h; ///< initial height that will be requested
		std::string title; ///< title of the window

		// actual size
		int win_width;
		int win_height;
	public:
		Window();
		~Window();
		void set_default_size(int w, int h); ///< set initial size that will be requested
		void set_title(std::string const & title); ///< set the title of window
		void resize(int width, int height); ///< change the size of mapped window
		void refresh();
		void reconfigure(); ///< ask to reconfigure the layout
		void make_current();
		void show();
		void hide();
		bool is_realized() const;
		int get_id() const;
	};
}
#endif
